# -----------------------------------------------------------------------------
# Programm         : G r a p h . p m
# -----------------------------------------------------------------------------
# Autor            : Christian Dhl
# Erstellt         : 08. Februar 2004
# Zuletzt gendert : 09. Februar 2004
# Aufgabe          : Logische Graphenklasse (ohne Darstellungselemente).
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Globale private Datei-Variablen:
# -----------------------------------------------------------------------------
my $Debug  = 2; # Debug level


# -----------------------------------------------------------------------------
# Klasse           : G r a p h
# abgeleitet von   : none
# -----------------------------------------------------------------------------
# Autor            : Christian Dhl
# Beschreibung     : Eine Graphen-Klasse.
# Attribute        : n           - Anzahl der Ecken
#                    m           - Anzahl der Kanten
#                    kantenform  - Art der Kante (normal, Pfeil hin...)
#                    KantenA     - anonymes Array mit Ecken am Kantenanfang
#                    KantenE     - anonymes Array mit Ecken am Kantenende
#                    Kantenform  - anonymes Array mit einer skalaren
#                                  Bezeichnung, die hier nicht weiter
#                                  inteessiert, sondern nur fr das Programm
#                                  interessant ist, welches diese Klasse
#                                  verwendet.
# Klassen-Methoden : new         - Erstellt ein Objekt dieser Klasse.
# Objekt-Methoden  : _property       - Hilfsfunktion fr Getter / Setter
#                    n               - Getter / Setter
#                    m               - Getter / Setter
#                    kantenform      - Getter / Setter
#                    KantenA         - Getter / Setter
#                    KantenE         - Getter / Setter
#                    Kantenform      - Getter / Setter
#                    ecke_einfuegen  - Ecke einfgen
#                    ecke_loeschen   - Ecke lschen
#                    ecke_mergen     - Ecke mergen (Ecke wird gelscht
#                                      und die Nachbarn der Ecke alle
#                                      miteinander durch Kanten verbunden).
#                    kante_existiert - Testet, ob die beiden angegeben Ecken
#                                      durch eine Kante verbunden sind.
#                    kante_einfuegen - Fgt eine Kante ein.
#                    kante_loeschen  - Lscht eine Kante.
#                    clear           - Lscht alles.
# -----------------------------------------------------------------------------
# 0.0.1 - 08.02.2004 - CD - erstellt
# -----------------------------------------------------------------------------
package Graph;

use strict;
use warnings;

use Data::Dumper;

sub new {
# -----------------------------------------------------------------------------
# Methode      : n e w
# Klasse       : Graph
# -----------------------------------------------------------------------------
# Autor        : Christian Dhl
# Aufgabe      : Erzeugt ein neues Graph-Objekt (oder ein Objekt einer Klasse,
#                die von dieser Klasse abgeleitet wurde und uns bittet, das
#                Objekt fr sie zu erzeugen).
# Parameter    : 1) Klasse
#                2) ev. gewnschte Klasse (im Fall einer abgeleiteten Klasse)
#                Und ggf. Attribute: siehe Klassenbeschreibung oben
# Rckgabewert : Das geblesste neue Objekt.
# -----------------------------------------------------------------------------
# 0.0.1 - 08.02.2004 - CD - erstellt
# -----------------------------------------------------------------------------

    my $class = shift; # Die Klasse, von der new aufgerufen wurde:
                       #     my $object = Graph->new(...);
                       # oder
                       #     my $object = new Graph(...);


    my $wantclass;     # Dies ist die Klasse, in welche das neue Objekt ge-
                       # blessed werden soll.
                       # Falls die Methode new mit einer gerade Anzahl an
                       # Parametern aufgerufen wurde (nachdem $class schon ent-
                       # nommen wurde), wurde keine gewnschte Klasse angege-
                       # ben. In diesem Fall verwenden wir $class.
                       # Anderenfalls verwenden wir die bergebene Klasse.

    my $n = scalar @_; # Bestimmt die Anzahl an verbliebenen Parametern.

    #--------------------------------------------------------------------------
    # Falls noch Parameter da sind und ihre Anzahl ungerade ist, dann mchte
    # der Aufrufer, dass wir das Objekt auf die gewnschte Klasse blessen, die
    # als nchster Parameter angegeben ist.
    # Also holen wir sie mit shift aus den Aufrufparametern und legen sie
    # in $wantclass ab.
    #--------------------------------------------------------------------------
    if ($n > 0 and ($n >> 1) << 1 != $n) {
        $wantclass = shift;
    }
    #--------------------------------------------------------------------------
    # Anderenfalls wurde keine gewnschte Klasse angegeben und wir verwenden
    # $class aus dem ersten Parameter:
    #--------------------------------------------------------------------------
    else {
        $wantclass = $class;
    }

    #--------------------------------------------------------------------------
    # Ausgabe der Aufrufparameter:
    #--------------------------------------------------------------------------
    print "Graph::new\n",
          "\tclass      = '$class'\n",
          "\twant class = '$wantclass'\n",
          "\tn          = $n\n",
          "\tParameter  = @_\n",
          if $Debug;

    #--------------------------------------------------------------------------
    # Wir erzeugen ein Objekt in der gewnschten Klasse (eine geblessde Refe-
    # renz auf einen ev. leeren anonymen Hash):
    #--------------------------------------------------------------------------
    my $self = bless({@_}, $wantclass);

    #--------------------------------------------------------------------------
    # Nun setzen wir noch ein paar Attribute:
    #--------------------------------------------------------------------------
    $self->n         (0);
    $self->m         (0);
    $self->kantenform('');
    $self->KantenA   ([]);
    $self->KantenE   ([]);
    $self->Kantenform([]);

    #--------------------------------------------------------------------------
    # ... und geben das erzeugte Objekt zurck:
    #--------------------------------------------------------------------------
    return $self;

} # sub Graph::new


# -----------------------------------------------------------------------------
# Methode      : _ p r o p e r t y
# Klasse       : Graph
# -----------------------------------------------------------------------------
# Autor        : Christian Dhl
# Aufgabe      : Private Funktion fr die einfache Definition von Gettern und
#                Settern.
# Parameter    : Attribut, Parameter
# Rckgabewert : Der aktuelle Wert (im Falle des Getters) oder der alte Wert
#                (im falle des Setters).
# -----------------------------------------------------------------------------
# 0.0.1 - 08.02.2004 - CD - erstellt
# -----------------------------------------------------------------------------
sub _property ($$;$) {
    my $self   = shift;    # Objekt
    my $attr   = shift;    # Attribute, das gesetzt oder geholt wird
    my $setter = @_ == 1;  # Ist die Methode Setter?
    my $value  = shift;    # Zu setzender Wert, falls Setter

    # Falls man hier "if (defined $value)" benutzen wrde, so knnte man keine
    # undefinierten Werte setzen, deshalb verwenden wir diese Form:
    if ($setter) {
        my $old_value = $self->{$attr};
        $self->{$attr} = $value;
        return $old_value;
    }
    else {
        return $self->{$attr};
    }
} # sub Graph::_property


# -----------------------------------------------------------------------------
# The getter/setter methods:
# -----------------------------------------------------------------------------
sub n          { return shift->_property('n',          @_) }
sub m          { return shift->_property('m',          @_) }
sub kantenform { return shift->_property('kantenform', @_) }
sub KantenA    { return shift->_property('KantenA',    @_) }
sub KantenE    { return shift->_property('KantenE',    @_) }
sub Kantenform { return shift->_property('Kantenform', @_) }


sub ecke_einfuegen {
# -----------------------------------------------------------------------------
# Methode      : e c k e _ e i n f u e g e n
# Klasse       : Graph
# -----------------------------------------------------------------------------
# Autor        : Christian Dhl
# Aufgabe      : Fgt eine neue Ecke in den Graphen ein.
# Parameter    : keine
# Rckgabewert : keiner
# -----------------------------------------------------------------------------
# 0.0.1 - 08.02.2004 - CD - erstellt
# -----------------------------------------------------------------------------

    my $self = shift;

    die "Graph::ecke_einfuegen mit Parametern aufgerufen!" if scalar @_;

    print "Graph::ecke_einfuegen\n" if $Debug;

    #--------------------------------------------------------------------------
    # Eckenzhler incrementieren
    #--------------------------------------------------------------------------
    $self->n($self->n()+1);

} # sub Graph::ecke_einfuegen


sub ecke_loeschen {
# -----------------------------------------------------------------------------
# Methode      : e c k e _ l o e s c h e n
# Klasse       : Graph
# -----------------------------------------------------------------------------
# Autor        : Christian Dhl
# Aufgabe      : Lscht die angegebene Ecke mit allen inzidierenden Kanten
#                aus dem Graphen.
# Parameter    : zu lschende Ecke
# Rckgabewert : keiner
# -----------------------------------------------------------------------------
# 0.0.1 - 08.02.2004 - CD - erstellt
# -----------------------------------------------------------------------------

    my $self = shift;

    die "Graph::ecke_loeschen falscher Anzahl Parametern aufgerufen!"
        if scalar @_ != 1;
    die "Graph::ecke_loeschen aufgerufen, aber der Graph hat keine Ecken!"
        unless $self->n() > 0;

    my $e = shift;


    print "Graph::ecke_loeschen($e)\n" if $Debug;

    die "Graph::ecke_loeschen mit falschem Eckenwert '$e' aufgerufen, n ist " .
        $self->n() ."!"
        if $e > $self->n();

    print Dumper($self) if $Debug > 1;

    #--------------------------------------------------------------------------
    # Kanten raussuchen, die zu dieser Ecke gehren und lschen:
    #--------------------------------------------------------------------------
    for my $i (reverse 0..$self->m()-1) {
        if ($self->KantenA()->[$i] == $e or $self->KantenE()->[$i] == $e) {
            $self->kante_loeschen($i);
            #print "lsche Kante $i ",
            #      $self->KantenA->[$i], " --- ", $self->KantenE()->[$i], "\n"
            #      if $Debug>1;
            #splice @{$self->KantenA()},    $i, 1;
            #splice @{$self->KantenE()},    $i, 1;
            #splice @{$self->Kantenform()}, $i, 1;
            #$self->m($self->m()-1);
        }
    }

    #--------------------------------------------------------------------------
    # Eckenzhler decrementieren
    #--------------------------------------------------------------------------
    $self->n($self->n()-1);

    #--------------------------------------------------------------------------
    # Korrigieren der hheren Eckennummern:
    #--------------------------------------------------------------------------
    for my $j (0..$self->m()-1) {
        $self->KantenA()->[$j] -= 1 if $self->KantenA()->[$j] >= $e;
        $self->KantenE()->[$j] -= 1 if $self->KantenE()->[$j] >= $e;
    }

    print Dumper($self) if $Debug > 1;
} # sub ecke_loeschen


sub ecke_mergen {
# -----------------------------------------------------------------------------
# Methode      : e c k e _ m e r g e n
# Klasse       : Graph
# -----------------------------------------------------------------------------
# Autor        : Christian Dhl
# Aufgabe      : Mergt die angegebene Ecke (d.h. die Ecke wird gelscht und die
#                Kanten zusammengezogen.
# Parameter    : Zu mergende Ecke
# Rckgabewert : keiner
# -----------------------------------------------------------------------------
# 0.0.1 - 08.02.2004 - CD - erstellt
# -----------------------------------------------------------------------------

    my $self = shift;

    die "Graph::ecke_mergen mit falscher Anzahl Parametern aufgerufen!"
        if scalar @_ != 1;

    my $e = shift;

    print "Graph::ecke_mergen($e)\n" if $Debug;

    #--------------------------------------------------------------------------
    # Ecke mergen:
    #--------------------------------------------------------------------------
    print Dumper($self) if $Debug > 1;

    #--------------------------------------------------------------------------
    # Kanten raussuchen, die zu dieser Ecke gehren:
    #--------------------------------------------------------------------------
    my @ecken = (); # Nachbarn von $e, die dann verbunden werden mssen
    my @kform = (); # Kantenformen, im Moment ignoriert

    for my $i (reverse 0..$self->m()-1) {
        if ($self->KantenA()->[$i] == $e or $self->KantenE()->[$i] == $e) {
            print "loesche Kante $i ",
                  $self->KantenA->[$i], " --- ", $self->KantenE()->[$i], "\n"
                  if $Debug>1;

            my $e1 = splice @{$self->KantenA()}, $i, 1;
            my $e2 = splice @{$self->KantenE()}, $i, 1;
            push @ecken, ($e1!=$e ? $e1 : $e2);
            push @kform, splice @{$self->Kantenform()}, $i, 1;
            # Bei der Kantenform msste man noch bercksichtigen, welche Ecke
            # die Kante $e war. Und dann bei Kantenpaaren mit der selben
            # Ausprgung diese wieder verwenden. Im Moment wird alles als
            # normale Kante erzeugt.

            $self->m($self->m()-1);
        }
    }

    print "Nachbarn der geloeschten Ecke: @ecken\n";

    #--------------------------------------------------------------------------
    # Neue Kanten eintragen:
    #--------------------------------------------------------------------------
    for my $i1 (0..$#ecken-1) {
        print "i1=$i1\n";
        for my $i2 ($i1+1..$#ecken) {
            print "i2=$i2\n";
            print "fuege Kante ein: $i1 --- $i2\n" if $Debug;
            $self->kante_einfuegen($ecken[$i1], $ecken[$i2])
                unless $self->kante_existiert($ecken[$i1], $ecken[$i2]);
        }
    }

    #--------------------------------------------------------------------------
    # Ecke lschen:
    #--------------------------------------------------------------------------
    $self->n($self->n()-1);

    #--------------------------------------------------------------------------
    # Korrigieren der hheren Eckennummern:
    #--------------------------------------------------------------------------
    for my $j (0..$self->m()-1) {
        $self->KantenA()->[$j] -= 1 if $self->KantenA()->[$j] >= $e;
        $self->KantenE()->[$j] -= 1 if $self->KantenE()->[$j] >= $e;
    }

    print Dumper($self) if $Debug > 1;

} # sub Graph::ecke_mergen


sub kante_existiert {
# -----------------------------------------------------------------------------
# Methode      : k a n t e _ e x i s t i e r t
# Klasse       : Graph
# -----------------------------------------------------------------------------
# Autor        : Christian Dhl
# Aufgabe      : Testet, ob die angegebene Kante existiert
# Parameter    : 1) Ecke Anfang
#                2) Ecke Ende
# Rckgabewert : undef falls nicht vorhanden, anderenfalls Kantentyp
# -----------------------------------------------------------------------------
# 0.0.1 - 08.02.2004 - CD - erstellt
# -----------------------------------------------------------------------------

    my $self = shift;

    die "Graph::kante_existiert mit falscher Anzahl Parametern aufgerufen!"
        if scalar @_ != 2;

    my $ea     = shift;
    my $ee     = shift;

    my $index  = -1;

    print "Graph::kante_existiert($ea, $ee)\n" if $Debug;

    for my $i (0..$self->m()-1) {
        if (
            $ea == $self->KantenA()->[$i] and $ee == $self->KantenE()->[$i] or
            $ea == $self->KantenE()->[$i] and $ee == $self->KantenA()->[$i]
           )
        {
            $index = $i;
            last;
        }
    }


    if ($index == -1) {
        return undef;
    }
    else {
        return $self->Kantenform()->[$index];
    }
} # sub Graph::kante_existiert


sub kante_einfuegen {
# -----------------------------------------------------------------------------
# Methode      : k a n t e _ e i n f u e g e n
# Klasse       : Graph
# -----------------------------------------------------------------------------
# Autor        : Christian Dhl
# Aufgabe      : Fgt eine Kante in den Graphen ein.
# Parameter    : 1) Startecke
#                2) Zielecke
#                3) Optional Kantenform, sonst $self->kantenform()
# Rckgabewert : keiner
# -----------------------------------------------------------------------------
# 0.0.1 - 08.02.2004 - CD - erstellt
# -----------------------------------------------------------------------------

    my $self = shift;

    die "Graph::kante_einfuegen falscher Anzahl Parametern aufgerufen!"
        if scalar @_ < 2 or scalar @_ > 3;
    die "Graph::kante_einfuegen aufgerufen, aber der Graph hat keine Ecken!"
        unless $self->n() > 0;

    my $ea = shift;
    my $ee = shift;
    my $kf = shift || $self->kantenform();


    print "Graph::kante_einfuegen($ea, $ee, $kf)\n" if $Debug;

    push @{$self->KantenA()},    $ea;
    push @{$self->KantenE()},    $ee;
    push @{$self->Kantenform()}, $kf;

    $self->m($self->m()+1);

} # sub Graph::kante_einfuegen


sub kante_loeschen {
# -----------------------------------------------------------------------------
# Methode      : k a n t e _ l o e s c h e n
# Klasse       : Graph
# -----------------------------------------------------------------------------
# Autor        : Christian Dhl
# Aufgabe      : Lscht die angegebene Kante aus dem Graphen.
# Parameter    : Kante
# Rckgabewert : keiner
# -----------------------------------------------------------------------------
# 0.0.1 - 08.02.2004 - CD - erstellt
# -----------------------------------------------------------------------------

    my $self = shift;

    die "Graph::kante_loeschen mit falscher Anzahl Parametern aufgerufen!"
        if scalar @_ != 1;

    my $k    = shift;

    print "Graph::kante_loeschen($k)\n" if $Debug;

    print "lsche Kante $k ",
          $self->KantenA->[$k], " --- ", $self->KantenE()->[$k], "\n"
          if $Debug>1;

    splice @{$self->KantenA()},    $k, 1;
    splice @{$self->KantenE()},    $k, 1;
    splice @{$self->Kantenform()}, $k, 1;

    $self->m($self->m()-1);

} # sub Graph::kante_loeschen


sub clear {
# -----------------------------------------------------------------------------
# Methode      : c l e a r
# Klasse       : Graph
# -----------------------------------------------------------------------------
# Autor        : Christian Dhl
# Aufgabe      : Lscht alle Ecken und Kanten.
# Parameter    : keine
# Rckgabewert : keiner
# -----------------------------------------------------------------------------
# 0.0.1 - 09.02.2003 - CD - Erstellt
# -----------------------------------------------------------------------------

    my $self = shift;

    die "Graph::clear mit Parametern aufgerufen!"
        if scalar @_ > 0;

    print "Graph::clear\n" if $Debug;


    #--------------------------------------------------------------------------
    # Ecken- und Knatenzahl auf Null setzen:
    #--------------------------------------------------------------------------
    $self->n(0);
    $self->m(0);

    #--------------------------------------------------------------------------
    # Speicherplatz freigeben:
    #--------------------------------------------------------------------------
    $self->KantenA   (undef);
    $self->KantenE   (undef);
    $self->Kantenform(undef);

    #--------------------------------------------------------------------------
    # neue leere anonyme Arrays anlegen:
    #--------------------------------------------------------------------------
    $self->KantenA   ([]);
    $self->KantenE   ([]);
    $self->Kantenform([]);
} # sub Graph2d::clear


return 1;

__END__

=head1 NAME

Graph.pm - a module for graphs


=head1 COPYRIGHT

Copyright 2004 Christian Duehl. All rights reserved.

This library is free software. You can redistribute it and/or
modify it under the same terms as perl itself.

=cut
